//   Copyright Naoki Shibata and contributors 2010 - 2020.
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

#ifndef __SLEEFQUAD_H__
#define __SLEEFQUAD_H__

#include "sleef.h"

#ifdef __cplusplus
extern "C"
{
#endif

#if (defined(__MINGW32__) || defined(__MINGW64__) || defined(__CYGWIN__) || defined(_MSC_VER)) && !defined(SLEEF_STATIC_LIBS)
#ifdef IMPORT_IS_EXPORT
#define IMPORT __declspec(dllexport)
#else // #ifdef IMPORT_IS_EXPORT
#define IMPORT __declspec(dllimport)
#if (defined(_MSC_VER))
#pragma comment(lib,"sleefquad.lib")
#endif // #if (defined(_MSC_VER))
#endif // #ifdef IMPORT_IS_EXPORT
#else // #if (defined(__MINGW32__) || defined(__MINGW64__) || defined(__CYGWIN__) || defined(_MSC_VER)) && !defined(SLEEF_STATIC_LIBS)
#define IMPORT
#endif // #if (defined(__MINGW32__) || defined(__MINGW64__) || defined(__CYGWIN__) || defined(_MSC_VER)) && !defined(SLEEF_STATIC_LIBS)

//

#if !defined(Sleef_quadx1_DEFINED)
#define Sleef_quadx1_DEFINED
typedef struct {
  uint64_t x, y;
} Sleef_quadx1;
#endif

#if !defined(Sleef_quadx2_DEFINED)

#if defined(__SSE2__)
#define Sleef_quadx2_DEFINED
typedef struct {
  __m128i x, y;
} Sleef_quadx2;
#endif

#if defined(__aarch64__)
#define Sleef_quadx2_DEFINED
typedef struct {
  uint32x4_t x, y;
} Sleef_quadx2;
#endif

#if defined(__VSX__)
#define Sleef_quadx2_DEFINED
typedef struct {
  __vector unsigned int x, y;
} Sleef_quadx2;
#endif

#if defined(__VX__) && (defined(__VECTOR_KEYWORD_SUPPORTED__) || defined(__VEC__))
#define Sleef_quadx2_DEFINED
typedef struct {
  __vector unsigned long long x, y;
} Sleef_quadx2;
#endif

#endif // #if !defined(Sleef_quadx2_DEFINED)

#if !defined(Sleef_quadx4_DEFINED)
#if defined(__AVX__)
#define Sleef_quadx4_DEFINED
typedef struct {
  __m256i x, y;
} Sleef_quadx4;
#endif
#endif

#if !defined(Sleef_quadx8_DEFINED)
#if defined(__AVX512F__)
#define Sleef_quadx8_DEFINED
typedef struct {
  __m512i x, y;
} Sleef_quadx8;
#endif
#endif


#if !defined(Sleef_svquad_DEFINED)
#if defined(__ARM_FEATURE_SVE)
#define Sleef_svquad_DEFINED
typedef svfloat64x2_t Sleef_svquad;
#endif
#endif

//

#if defined(__BYTE_ORDER__) && (__BYTE_ORDER__ == __ORDER_BIG_ENDIAN__)
#define SLEEF_Q(H, L, E) (						\
      (union {								\
	struct {							\
	  uint64_t h, l;						\
	};								\
	Sleef_quad q;							\
      }) {								\
        .h = (((int64_t)(H) < 0 ? 1ULL : 0ULL) << 63) |			\
             ((0x7fff & (uint64_t)((E) + 16383)) << 48) |		\
	     ((int64_t)(H) < 0 ? -(int64_t)(H) : (int64_t)(H) & 0xffffffffffffULL), \
	.l = (uint64_t)(L)}.q)
#else
#define SLEEF_Q(H, L, E) (						\
      (union {								\
	struct {							\
	  uint64_t l, h;						\
	};								\
	Sleef_quad q;							\
      }) {								\
        .h = (((int64_t)(H) < 0 ? 1ULL : 0ULL) << 63) |			\
             ((0x7fff & (uint64_t)((E) + 16383)) << 48) |		\
	     ((int64_t)(H) < 0 ? -(int64_t)(H) : (int64_t)(H) & 0xffffffffffffULL), \
	.l = (uint64_t)(L)}.q)
#endif

//

#include <stdio.h>
#include <stdarg.h>

IMPORT Sleef_quad Sleef_strtoq(const char *str, char **endptr);

IMPORT int Sleef_fprintf(FILE *fp, const char *fmt, ...);
IMPORT int Sleef_vfprintf(FILE *fp, const char *fmt, va_list ap);
IMPORT int Sleef_printf(const char *fmt, ...);
IMPORT int Sleef_vprintf(const char *fmt, va_list ap);
IMPORT int Sleef_snprintf(char *str, size_t size, const char *fmt, ...);
IMPORT int Sleef_vsnprintf(char *str, size_t size, const char *fmt, va_list ap);

IMPORT int Sleef_registerPrintfHook();
IMPORT void Sleef_unregisterPrintfHook();

//

#ifdef __STDC__
IMPORT CONST Sleef_quad Sleef_addq1_u05purec(Sleef_quad, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_subq1_u05purec(Sleef_quad, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_mulq1_u05purec(Sleef_quad, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_divq1_u05purec(Sleef_quad, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_negq1_purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_sqrtq1_u05purec(Sleef_quad);
IMPORT CONST int32_t Sleef_icmpltq1_purec(Sleef_quad, Sleef_quad);
IMPORT CONST int32_t Sleef_icmpleq1_purec(Sleef_quad, Sleef_quad);
IMPORT CONST int32_t Sleef_icmpgtq1_purec(Sleef_quad, Sleef_quad);
IMPORT CONST int32_t Sleef_icmpgeq1_purec(Sleef_quad, Sleef_quad);
IMPORT CONST int32_t Sleef_icmpeqq1_purec(Sleef_quad, Sleef_quad);
IMPORT CONST int32_t Sleef_icmpneq1_purec(Sleef_quad, Sleef_quad);
IMPORT CONST int32_t Sleef_icmpq1_purec(Sleef_quad, Sleef_quad);
IMPORT CONST int32_t Sleef_iunordq1_purec(Sleef_quad, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_iselectq1_purec(int32_t, Sleef_quad, Sleef_quad);
IMPORT CONST double Sleef_cast_to_doubleq1_purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_cast_from_doubleq1_purec(double);
IMPORT CONST int64_t Sleef_cast_to_int64q1_purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_cast_from_int64q1_purec(int64_t);
IMPORT CONST uint64_t Sleef_cast_to_uint64q1_purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_cast_from_uint64q1_purec(uint64_t);
IMPORT CONST Sleef_quad Sleef_loadq1_purec(Sleef_quad *);
IMPORT CONST void Sleef_storeq1_purec(Sleef_quad *, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_getq1_purec(Sleef_quad, int);
IMPORT CONST Sleef_quad Sleef_setq1_purec(Sleef_quad, int, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_splatq1_purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_sinq1_u10purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_cosq1_u10purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_tanq1_u10purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_asinq1_u10purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_acosq1_u10purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_atanq1_u10purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_expq1_u10purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_exp2q1_u10purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_exp10q1_u10purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_expm1q1_u10purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_logq1_u10purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_log2q1_u10purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_log10q1_u10purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_log1pq1_u10purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_fabsq1_purec(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_copysignq1_purec(Sleef_quad, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_fmaxq1_purec(Sleef_quad, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_fminq1_purec(Sleef_quad, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_fdimq1_u05purec(Sleef_quad, Sleef_quad);
#endif
#ifdef FP_FAST_FMA
IMPORT CONST Sleef_quad Sleef_addq1_u05purecfma(Sleef_quad, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_subq1_u05purecfma(Sleef_quad, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_mulq1_u05purecfma(Sleef_quad, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_divq1_u05purecfma(Sleef_quad, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_negq1_purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_sqrtq1_u05purecfma(Sleef_quad);
IMPORT CONST int32_t Sleef_icmpltq1_purecfma(Sleef_quad, Sleef_quad);
IMPORT CONST int32_t Sleef_icmpleq1_purecfma(Sleef_quad, Sleef_quad);
IMPORT CONST int32_t Sleef_icmpgtq1_purecfma(Sleef_quad, Sleef_quad);
IMPORT CONST int32_t Sleef_icmpgeq1_purecfma(Sleef_quad, Sleef_quad);
IMPORT CONST int32_t Sleef_icmpeqq1_purecfma(Sleef_quad, Sleef_quad);
IMPORT CONST int32_t Sleef_icmpneq1_purecfma(Sleef_quad, Sleef_quad);
IMPORT CONST int32_t Sleef_icmpq1_purecfma(Sleef_quad, Sleef_quad);
IMPORT CONST int32_t Sleef_iunordq1_purecfma(Sleef_quad, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_iselectq1_purecfma(int32_t, Sleef_quad, Sleef_quad);
IMPORT CONST double Sleef_cast_to_doubleq1_purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_cast_from_doubleq1_purecfma(double);
IMPORT CONST int64_t Sleef_cast_to_int64q1_purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_cast_from_int64q1_purecfma(int64_t);
IMPORT CONST uint64_t Sleef_cast_to_uint64q1_purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_cast_from_uint64q1_purecfma(uint64_t);
IMPORT CONST Sleef_quad Sleef_loadq1_purecfma(Sleef_quad *);
IMPORT CONST void Sleef_storeq1_purecfma(Sleef_quad *, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_getq1_purecfma(Sleef_quad, int);
IMPORT CONST Sleef_quad Sleef_setq1_purecfma(Sleef_quad, int, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_splatq1_purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_sinq1_u10purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_cosq1_u10purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_tanq1_u10purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_asinq1_u10purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_acosq1_u10purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_atanq1_u10purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_expq1_u10purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_exp2q1_u10purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_exp10q1_u10purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_expm1q1_u10purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_logq1_u10purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_log2q1_u10purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_log10q1_u10purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_log1pq1_u10purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_fabsq1_purecfma(Sleef_quad);
IMPORT CONST Sleef_quad Sleef_copysignq1_purecfma(Sleef_quad, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_fmaxq1_purecfma(Sleef_quad, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_fminq1_purecfma(Sleef_quad, Sleef_quad);
IMPORT CONST Sleef_quad Sleef_fdimq1_u05purecfma(Sleef_quad, Sleef_quad);
#endif
#ifdef __cplusplus
}
#endif

#undef IMPORT
#endif // #ifndef __SLEEFQUAD_H__
